		TITLE	'Listato 103'
		NAME	AlphaModeSet
		PAGE	55,132

;
; Nome:		AlphaModeSet
;
;		Programma il CRTC nelle modalit alfanumeriche a 80 colonne EGA
;
; Chiamante:	Microsoft C:
;
;			azzera AlphaModeSet(w,h,c);
;
;			int	w;	/* larghezza della matrice di carattere */
;			int	h;	/* altezza della matrice di carattere */
;			int	c;	/* dimensione del codice di carattere */
;

ARGw		EQU	byte ptr [bp+4]		; deve essere larga 8 o 9 pixel
ARGh		EQU	byte ptr [bp+6]		; deve essere alta da 2 a 32 pixel
ARGc		EQU	byte ptr [bp+8]		; deve essere 8 o 9 bit

CRT_MODE	EQU	49h		; indirizzi nell'area dati di visualizzazione del BIOS
CRT_COLS	EQU	4Ah
ADDR_6845	EQU	63h

DGROUP		GROUP	_DATA

_TEXT		SEGMENT	byte public 'CODE'
		ASSUME	cs:_TEXT,ds:DGROUP

		PUBLIC	_AlphaModeSet
_AlphaModeSet	PROC	near

		push	bp		; mantiene registri del chiamante
		mov	bp,sp
		push	si

; programma il CRTC

		mov	bx,40h
		mov	es,bx		; ES := segmento dati BIOS del video

		mov	bl,ARGw		; BL := larghezza carattere
		mov	bh,ARGh		; BH := altezza carattere
		call	SetCRTC

; programma il sequencer ed il controller di attributo per 8 o 9 punti per carattere

		mov	dx,3C4h
		mov	ax,0100h	; AH bit 1 := 0 (reset sincrono)
					; AL := 0 (numero registro reset)
		cli			; disabilita interrupt
		out	dx,ax		; reset sincrono del sequencer

		mov	bx,1		; BH,BL := valori per caratteri larghi 8:
					;  BH := 0 (valore per Scorr. Pixel oriz)
					;  BL := 1 (valore per modalit sincronizzazione)
		cmp	ARGw,8
		je	L01		; salta se caratteri larghi 8

		mov	bx,0800h	; BH,BL := valori per caratteri larghi 9

L01:		mov	ah,bl		; AH := valore per reg modalit sincronizzazione
		mov	al,1		; AL := numero reg modalit sincronizzazione
		out	dx,ax		; programma il sequencer

		mov	ax,0300h	; AH := 3 (disabilita ripristino)
					; AL := 0 (numero registro sequencer)
		out	dx,ax		; disabilita reset sequencer
		sti			; abilita interrupt

		mov	bl,13h		; BL := numero registro Scorr. Pixel orizzontale
		mov	ax,1000h	; AH := 10H (numero funzione INT 10H)
					; AL := 0 (imposta registro specificato)
		int	10h		; programma controller attributo

; programma il controller di attributo per codici di carattere da 8 o 9 bit

		mov	ax,1000h	; AH := 10H (numero funzione INT 10H)
					; AL := 0 (imposta registro specificato)
		mov	bx,0F12h	; BH := 0FH (valore abilitazione piano colore)
					; BL := 12H (# reg abilitazione piano colore)
		cmp	ARGc,8
		je	L02		; salta se codici carattere a 8 bit

		mov	bh,7		; BH bit 3 := 0 (ignora bit 3 di tutti
					;  gli attributi)
L02:		int	10h		; aggiorna registro abilitazione piano colore

; aggiorna area dati di visualizzazione BIOS

		cmp	byte ptr es:[CRT_MODE],7
		jne	L03		; salta se non  modalit monocromatica

		mov	ax,720		; AX := pixel visualizzati per riga
		div	ARGw		; AL := colonne di caratteri visualizzate
		mov	es:[CRT_COLS],al

L03:		pop	si
		pop	bp
		ret

_AlphaModeSet	ENDP


SetCRTC         PROC	near		; Chiamante:	BH = altezza carattere
					;		BL = larghezza carattere

		push	dx
		mov	dx,es:[ADDR_6845]  ; porta di I/O del CRTC

; stabilisce temporizzazione veriticale del CRTC e posizione del cursore nella matrice di carattere

		push	bx		; mantiene altezza e larghezza
		mov	ax,1110h	; AH := 11H (numero funzione INT 10H)
					; AL := 0 (alfabeto utente caricato)
		xor	cx,cx		; CX := 0 (non memorizza alcun carattere)
		int	10h		; richiama BIOS per programmare il CRTC
					;  per dimensione verticale dei caratteri

		pop	ax		; AH := altezza caratteri
		push	ax		; mantiene altezza e larghezza
		sub	ah,2		; AH := linea di scansione iniziale del cursore
		mov	al,0Ah		; AL := 0AH (numero reg cursore inizio)
		out	dx,ax		; aggiorna registro cursore inizio del CRTC

		mov	ax,000Bh	; AH := 0 (valore cursore fine)
					; AL := 0BH (numero reg cursore fine)
		out	dx,ax		; aggiorna registro cursore fine del CRTC

; stabilisce temporizzazione orizzontale del CRTC

		pop	bx		; BX := altezza e larghezza carattere
		cmp	byte ptr es:[CRT_MODE],7
		jne	L10		; esce se non  modalit monocromatica

		xor	bh,bh		; BX := larghezza carattere
		sub	bl,8		; BX := 0 o 1
		neg	bx		; BX := 0 o 0FFFFH
		and	bx,14		; BX := 0 o 14 (offset nella tabella)
		mov	si,bx		; SI := offset nella tabella

		add	si,offset DGROUP:HorizParms	; DS:SI -> parametri
		call	UpdateCRTC

L10:		pop	dx
		ret

SetCRTC         ENDP


UpdateCRTC	PROC	near		; Chiamante:	DX = porta indirizzo CRTC
					;		DS:SI -> parametri
					; distrugge:	AX,CX

		mov	cx,7		; CX := numero registri da aggiornare
		
L20:		lodsw			; AH := dati per registro del CRTC in AL
		out	dx,ax		; aggiorna il registro
		loop	L20

		ret

UpdateCRTC	ENDP

_TEXT		ENDS


_DATA		SEGMENT	word public 'DATA'

HorizParms	DW	6C00h,5901h,6002h,2403h,5B04h,6A05h,2D13h  ; larg. 8
		DW	6000h,4F01h,5602h,3A03h,5104h,6005h,2813h  ; larg. 9

_DATA		ENDS

		END
