		TITLE	'Listato 105'
		NAME	AlphaModeSet
		PAGE	55,132

;
; Nome:		AlphaModeSet
;
; Funzione:	Programma il CRTC nelle modalit alfanumeriche su HGC+ o scheda InColor
;
; Chiamante:	Microsoft C:
;
;			azzera AlphaModeSet(w,h,c);
;
;			int	w;	/* larghezza della matrice del carattere */
;			int	h;	/* altezza della matrice di carattere */
;			int	c;	/* dimensione del codice di carattere */
;

ARGw		EQU	byte ptr [bp+4]		; deve essere larga 8 o 9 pixel
ARGh		EQU	byte ptr [bp+6]		; deve essere alta da 4 a 16 pixel
ARGc		EQU	byte ptr [bp+8]		; deve essere 8 o 12 bit

CRT_COLS	EQU	4Ah
CRT_LEN		EQU	4Ch
CRT_MODE_SET	EQU	65h
ROWS		EQU	84h

DGROUP		GROUP	_DATA

_TEXT		SEGMENT	byte public 'CODE'
		ASSUME	cs:_TEXT,ds:DGROUP

		PUBLIC	_AlphaModeSet
_AlphaModeSet	PROC	near

		push	bp		; mantiene registri del chiamante
		mov	bp,sp
		push	ds
		push	si

; Imposta commutatore configurazione per portare RAM a partire da B000:4000 nella mappa di memoria

		mov	dx,3BFh		; DX := porta commutatore configurazione
		mov	al,1		; AL bit 1 := 0 (esclude secondi 32K
					;  del buffer del video)
					; AL bit 0 := 1 (rende indirizzabile RAM
		out	dx,ax		;   a B000:4000)

; Cancella lo schermo per evitare interferenze durante la programmazione del CRTC

		mov	dx,3B8h		; DX := porta reg. di controllo modalit CRTC
		xor	al,al		; AL bit 3 := 0 (disabilita segnale video)
		out	dx,al		; cancella lo schermo

; Programma il CRTC

		mov	bh,ARGw		; BH := larghezza carattere
		mov	bl,ARGh		; BL := altezza carattere
		call	SetHercCRTC

; Imposta xModeReg (registro Modalit-x)

		mov	dx,3B4h		; DX := porta indirizzo CRTC
		mov	ax,114h		; AH bit 0 := 1 (abilita generatore
					;  caratteri basato su RAM)
					; AL := 14h (numero xModeReg)
		cmp	ARGw,9
		je	L01		; salta se caratteri sono larghi 9

		or	ah,2		; AH bit 1 := 1 (caratteri larghi 8)

L01:		cmp	ARGc,8
		je	L02		; salta se codici di carattere sono a 8 bit

		or	ah,4		; AH bit 2 := 1 (codici di carattere a 12 bit)

L02:		out	dx,ax		; aggiorna il registro

; aggiorna area dati del BIOS del video

		mov	ax,40h
		mov	ds,ax		; DS := segmento dati del BIOS del video

		mov	ax,720		; AX := pixel visualizzati per riga
		div	ARGw		; AL := colonne di carattere visualizzate
		mov	ds:[CRT_COLS],al

		mov	ax,350		; AX := numero di linee di scansione visualizzate
		div	ARGh		; AL := righe di caratteri visualizzate
		dec	al		; AL := (righe di caratteri) - 1
		mov	ds:[ROWS],al

		inc	al
		mul	byte ptr ds:[CRT_COLS]
		shl	ax,1		; AX := righe * colonne * 2
		mov	ds:[CRT_LEN],ax

; riabilita schermo ed esce

		mov	dx,3B8h		; DX := porta controllo modalit del CRT
		mov	al,ds:[CRT_MODE_SET]	; ripristina valore precedente
		out	dx,al

		pop	si
		pop	ds
		pop	bp
		ret

_AlphaModeSet	ENDP


SetHercCRTC	PROC	near		; Chiamante :	BH = larghezza carattere
					;		BL = altezza carattere

		push	dx
		mov	dx,3B4h		; DX := porta reg indirizzo del CRTC 3B4h

; stabilisce posizione cursore nella matrice del carattere

		mov	ah,bl
		dec	ah		; AH := valore per reg linea di scansione massima
		mov	al,9		; AL := numero registro linea di scansione max
		out	dx,ax

		mov	al,0Bh		; AL := numero reg cursore fine
		out	dx,ax		; imposta cursore a fine ultima linea
					;  della matrice di carattere

		sub	ax,101h		; AH := da seconda a ultima linea
					; AL := 0AH (numero reg cursore inizio)
		out	dx,ax		; imposta cursore a inizio su
					;  seconda a ultima linea

; calcola offset nelle tabelle dei parametri

		sub	bx,0804h	; BH := 0 o 1
					; BL := da 0 a 12
		add	bx,bx
		add	bx,bx		; BH := 0 o 4
					; BL := da 0 a 48
; stabilisce temporizzazione orizzontale del CRTC

		push	bx		; mantiene BX
		mov	bl,bh
		xor	bh,bh		; BX := 0 o 4
		add	bx,offset DGROUP:HorizParms	; DS:BX -> parametri

		mov	al,0		; AL := primo reg del CRTC da aggiornare
		call	UpdateCRTC

; stabilisce temporizzazione verticale

		pop	bx
		xor	bh,bh		; BX := da 0 a 48
		add	bx,offset DGROUP:VertParms	; DS:BX -> parametri

		mov	al,4		; AL := primo reg del CRTC da aggiornare
		call	UpdateCRTC

		pop	dx		; ripristina DX
		ret

SetHercCRTC	ENDP


UpdateCRTC	PROC	near		; Chiamante:	AL = numero primo reg
					;		DX = porta indirizzo CRTC
					;		DS:BX -> parametri
					; Distrugge:	AX,CX

		mov	cx,4		; CX := numero di registri da aggiornare
		
L10:		mov	ah,[bx]		; AH := dati per registro del CRTC in AL
		out	dx,ax		; aggiorna il registro
		inc	ax		; AL := numero registro successivo
		inc	bx		; DS:BX -> valore successivo nella tabella
		loop	L10

		ret

UpdateCRTC	ENDP

_TEXT		ENDS


_DATA		SEGMENT	word public 'DATA'

HorizParms	DB	6Dh,5Ah,5Ch,0Fh		; larg. 8 pixel
		DB	61h,50h,52h,0Fh		; larg. 9 pixel


VertParms	DB	5Ch,02h,58h,59h		; alta 4 linee di scansione
		DB	4Ah,00h,46h,46h		;      5
		DB	3Dh,04h,3Ah,3Bh		;      6
		DB	34h,06h,32h,33h		;      7
		DB	2Dh,02h,2Bh,2Ch		;      8
		DB	28h,01h,26h,27h		;      9
		DB	24h,00h,23h,23h		;      10
		DB	20h,07h,1Fh,20h		;      11
		DB	1Dh,0Ah,1Dh,1Dh		;      12
		DB	1Bh,06h,1Ah,1Bh		;      13
		DB	19h,06h,19h,19h		;      14
		DB	17h,0Ah,17h,17h		;      15
		DB	16h,02h,15h,16h

_DATA		ENDS

		END
