		TITLE	'Listato 110'
		NAME	GetBitBlock10
		PAGE	55,132

;
; Nome:		GetBitBlock10
;
; Funzione:	Copia un blocco di bit dal buffer del video alla RAM di sistema
;		 nelle modalit grafiche EGA e VGA originali
;
; Chiamante:	Microsoft C:
;
;			int GetBitBlock10(x0,y0,x1,y1,buf);
;
;			     int x0,y0;    /* angolo superiore sinistro del blocco di bit */
;			     int x1,y1;       /* angolo inferiore destro */
;			     char far *buf;   /* buffer */
;
; Nota:		Ritorna dimensione del blocco di bit nella RAM di sistema.
;


ARGx0		EQU	word ptr [bp+4]
ARGy0		EQU	word ptr [bp+6]
ARGx1		EQU	word ptr [bp+8]
ARGy1		EQU	word ptr [bp+10]
ADDRbuf		EQU	         [bp+12]

VARPixelRows	EQU	word ptr [bp-2]
VARPixelRowLen	EQU	word ptr [bp-4]

BytesPerRow	EQU	80
ByteOffsetShift	EQU	3		; rispecchia numero di pixel per byte


_TEXT		SEGMENT	byte public 'CODE'
		ASSUME	cs:_TEXT

		EXTRN	PixelAddr10:near

		PUBLIC	_GetBitBlock10
_GetBitBlock10	PROC	near

		push	bp		; mantiene registri del chiamante
		mov	bp,sp
		sub	sp,4		; stabilisce cornice di stack
		push	ds
		push	si
		push	di

; calcola dimensioni del blocco di bit

		mov	ax,ARGx1
		sub	ax,ARGx0
		mov	cx,0FF07h	; CH := maschera di bit non spostata
					; CL := esegue AND della maschera per AL
		and	cl,al		; CL := numero di pixel nell'ultimo
					;  byte della riga
		xor	cl,7		; CL := numero di bit da spostare
		shl	ch,cl		; CH := maschera di bit per ultimo byte della riga
		mov	cl,ch
		push	cx		; salva su stack

		mov	cl,ByteOffsetShift
		shr	ax,cl
		inc	ax		; AX := numero di byte per riga
		push	ax		; salva su stack

		mov	ax,ARGy1
		sub	ax,ARGy0
		inc	ax		; AX := numero di righe di pixel
		push	ax		; salva su stack

; stabilisce indirizzamento

		mov	ax,ARGy0
		mov	bx,ARGx0
		call	PixelAddr10	; ES:BX -> x0,y0 nel buffer del video
		xor	cl,7		; CL := numero di bit da spostare a sinistra
		push	es
		pop	ds
		mov	si,bx		; DS:SI -> buffer nella RAM di sistema

; crea intestazione a 5 byte del blocco di bit

		pop	ax
		mov	VARPixelRows,ax
		stosw			; byte 0-1 := numero di righe di pixel
		pop	ax
		mov	VARPixelRowLen,ax
		stosw			; byte 2-3 := byte per riga di pixel
		pop	ax
		mov	ch,al		; CH := maschera di bit per ultimo byte nella riga
		stosb			; byte 4 := maschera di bit per ultimo byte

; imposta controller grafico

		mov	dx,3CEh		; DX := porta indirizzo controller grafico

		mov	ax,0005		; AH := 0 (modalit di lettura 0, modalit di scrittura 0)
					; AL := 5 (numero registro modalit)
		out	dx,ax		; imposta modalit di lettura 0

		mov	ax,0304h	;  AH := 3 (primo piano di bit da leggere)
					; AL := 4 (numero reg selez. mappa di lettura)

; copia dal buffer del video alla RAM di sistema

L01:		out	dx,ax		; seleziona succ. mappa di memoria da leggere
		push	ax		; mantiene numero mappa memoria
		push	VARPixelRows	; mantiene numero di righe di pixel
		push	si		; mantiene offset di x0,y0

L02:		mov	bx,VARPixelRowLen
		push	si		; mantiene SI all'inizio della riga di pixel

L03:		lodsw			; AL := byte successivo nel buffer del video
					; AH := (byte successivo) + 1
		dec	si		; DS:SI -> (byte successivo) + 1
		rol	ax,cl		; AL := successivi 4 pixel nella riga
		stosb			; copia sulla RAM di sistema
		dec	bx		; loop sulla riga
		jnz	L03

		and	es:[di-1],ch	; maschera ultimo byte di riga
		pop	si		; DS:SI -> inizio di riga
		add	si,BytesPerRow	; DS:SI -> inizio di riga successiva

		dec	VARPixelRows
		jnz	L02		; loop verso il basso sulle righe

		pop	si		; DS:SI -> inizio blocco di bit
		pop	VARPixelRows	; ripristina numero di righe di pixel
		pop	ax		; AH := ultima lettura di mappa
					; AL := numero reg selezione mappa lettura
		dec	ah
		jns	L01		; loop su piani di bit

		mov	ax,di
		sub	ax,ADDRbuf	; AX := ritorna valore (dimens. blocco di bit
					;  nella RAM di sistema)

		pop	di		; ripristina registri ed esce
		pop	si
		pop	ds
		mov	sp,bp
		pop	bp
		ret

_GetBitBlock10	ENDP

_TEXT		ENDS

		END
