		TITLE	'Listato  111'
		NAME	StoreBitBlock10
		PAGE	55,132

;
; Nome:		StoreBitBlock10
;
; Funzione:	Copia un blocco di bit dal buffer del video alla RAM di sistema
;		 nelle modalit grafiche EGA e VGA originali
;
; Chiamante:	Microsoft C:
;
;			azzera StoreBitBlock10(buf,x,y);
;
;			     char far *buf;   /* buffer */
;			     int x,y;    /* angolo superiore sinistro del blocco di bit */
;


ADDRbuf		EQU	dword ptr [bp+4]
ARGx		EQU	word ptr [bp+8]
ARGy		EQU	word ptr [bp+10]

VARPixelRows	EQU	word ptr [bp-2]
VARPixelRowLen	EQU	word ptr [bp-4]
VARRowCounter	EQU	word ptr [bp-6]
VARStartMask	EQU	word ptr [bp-8]
VAREndMaskL	EQU	word ptr [bp-10]
VAREndMaskR	EQU	word ptr [bp-12]

BytesPerRow	EQU	80		; larghezza logica del buffer del video
ByteOffsetShift	EQU	3		; rispecchia numero di pixel per byte
RMWbits		EQU	18h		; seleziona sostituzione, XOR, AND, o OR

_TEXT		SEGMENT	byte public 'CODE'
		ASSUME	cs:_TEXT

		EXTRN	PixelAddr10:near

		PUBLIC	_StoreBitBlock10
_StoreBitBlock10 PROC	near

		push	bp		; mantiene registri del chiamante
		mov	bp,sp
		sub	sp,12		; stabilisce cornice di stack
		push	ds
		push	si
		push	di

; stabilisce indirizzamento

		mov	ax,ARGy
		mov	bx,ARGx
		call	PixelAddr10	; ES:BX -> offset di byte di x,y
		inc	cl
		and	cl,7		; CL := numero di bit da spostare a sinistra

		mov	di,bx		; ES:DI -> x,y nel buffer del video

		lds	si,ADDRbuf	; ES:DI -> buffer nella RAM del sistema

; ricava dimensioni del blocco di bit dall'intestazione

		lodsw			; AX := numero di righe di pixel
		mov	VARPixelRows,ax
		lodsw			; AX := byte per riga di pixel
		mov	VARPixelRowLen,ax
		lodsb			; AL := maschera di bit per ultimo byte nella riga
		mov	ch,al

; imposta controller grafico

		mov	dx,3CEh		; DX := porta di I/O del controller grafico

		mov	ah,RMWbits	; AH := valore per registro rotazione
		mov	al,3		;  dati/selezione funzione
		out	dx,ax		; aggiorna questo registro

		mov	ax,0805h	; AH := 8 (modalit lettura 1, modalit scrittura 0)
					; AL := 5 (numero registro modalit)
		out	dx,ax		; imposta modalit lettura 0

		mov	ax,0007		; AH := 0 (ininfuente per tutte le mappe;
					;  letture di CPU ritornano sempre 0FFH)
					; AL := 7 (numero reg colore ininfluente)
		out	dx,ax		; imposta reg colore ininfluente

		mov	ax,0FF08h	; AH := 0FFH (valore per reg maschera di bit)
		out	dx,ax		; imposta reg maschera di bit

		mov	dl,0C4h		; DX := 3C4H (porta di I/O del sequencer)
		mov	ax,0802h	; AH := 1000B (valore per reg maschera di mappa)
					; AL := 2 (numero registro maschera di mappa)

		cmp	cx,0FF00h	; se maschera <> 0FFH o bit da spostare <> 0
		jne	L15		;  salta se non allineato al byte


; routine per blocchi di bit allineati al byte

		mov	cx,VARPixelRowLen

L10:		out	dx,ax		; abilita un piano di bit per scritture
		push	ax		; mantiene valore di maschera di mappa
		push	di		; mantiene offset di buffer del video di x,y
		mov	bx,VARPixelRows

L11:		push	di		; mantiene DI e CX
		push	cx

L12:		lodsb			; AL := successivo byte di pixel
		and	es:[di],al	; aggiorna piano di bit
		inc	di
		loop	L12
		
		pop	cx		; ripristina DI e CX
		pop	di	
		add	di,BytesPerRow	; ES:DI -> successiva riga di pixel nel buffer
		dec	bx
		jnz	L11		; loop verso il basso su righe di pixel

		pop	di		; ES:DI -> offset del buffer del video di x,y
		pop	ax		; AH := valore corrente reg maschera di mappa
		shr	ah,1		; AH := nuovo valore di maschera di mappa
		jnz	L10		; loop su tutti i piani di bit

		jmp	Lexit

; routine per blocchi di bit non allineati

L15:		push	ax		; mantiene valori reg maschera di mappa

		mov	bx,0FFh		; BH := 0 (maschera per primo byte nella riga)
					; BL := 0FFh
		mov	al,ch		; AL := maschera per ultimo byte nella riga di pixel
		cbw			; AH := 0FFh (maschera per ultimo byte -1)

		cmp	VARPixelRowLen,1
		jne	L16		; salta se pi di un byte per riga

		mov	bl,ch
		mov	ah,ch		; AH := maschera per ultimo byte -1
		xor	al,al		; AL := 0 (maschera per ultimo byte)

L16:		shl	ax,cl		; sposta maschere in posizione
		shl	bx,cl

		mov	bl,al		; salva maschere insieme a ..
		mov	al,8		; .. numero registro maschera di bit
		mov	VAREndMaskL,ax
		mov	ah,bl
		mov	VAREndMaskR,ax
		mov	ah,bh		
		mov	VARStartMask,ax
		
		mov	bx,VARPixelRowLen
		pop	ax		; ripristina valori reg maschera di mappa

; imposta pixel riga per riga nei piani di bit

L17:		out	dx,ax		; abilita un piano di bit per scritture
		push	ax		; mantiene valore di maschera di mappa
		push	di		; mantiene offset del buffer del video di x,y
		mov	dl,0CEh		; DX := 3CEH (porta controller grafico)

		mov	ax,VARPixelRows
		mov	VARRowCounter,ax  ; inizializza contatore di loop

; imposta pixel all'inizio di riga nel piano di bit correntemente abilitato

L18:		push	di		; mantiene offset dell'inizio della riga di pixel
		push	si		; mantiene offset della riga nel blocco di bit
		push	bx		; mantiene byte per riga di pixel

		mov	ax,VARStartMask
		out	dx,ax		; imposta reg maschera di bit per primo byte della riga

		lodsw			; AH := secondo byte di pixel
					; AL := primo byte di pixel
		dec	si		; DS:SI -> secondo byte di pixel
		test	cl,cl
		jnz	L19		; salta se non allineato a sinistra

		dec	bx		; BX := byte per riga - 1
		jnz	L20		; salta se almeno 2 byte per riga
		jmp	short L22	; salta se solo un byte per riga

L19:		rol	ax,cl		; AH := parte sinistra del primo byte,
					;       parte destra del secondo byte
					; AL := parte destra del primo byte,
					;       parte sinistra del secondo byte
		and	es:[di],ah	; imposta pixel per parte sinistra primo byte
		inc	di
		dec	bx		; BX := byte per riga - 2

L20:		push	ax		; mantiene pixel
		mov	ax,0FF08h
		out	dx,ax		; imposta reg maschera di bit per byte succ.
		pop	ax

		dec	bx
		jng	L22		; salta se solo 1 o 2 byte nella riga di pixel

; imposta pixel nel centro della riga

L21:		and	es:[di],al	; imposta pixel nella parte destra del byte corrente
		inc	di		;  e nella parte sinistra del byte successivo

		lodsw			; AH := successivo byte +1 di pixel
		dec	si		; AL := byte successivo di pixel
		rol	ax,cl		; AH := parte sinistra del byte successivo, parte
					;       destra del byte successivo +1
					; AL := parte destra del byte successivo, parte
					;       sinistra del byte successivo +1
		dec	bx
		jnz	L21		; loop su righe di pixel

; imposta pixel alla fine della riga

L22:		mov	bx,ax		; BH := parte destra dell'ultimo byte, parte
					;       sinistra dell'ultimo byte -1
					; BL := parte sinistra dell'ultimo byte, parte
					;       destra dell'ultimo byte -1
		mov	ax,VAREndMaskL	; AH := maschera per ultimo byte -1
					; AL := numero reg maschera di bit
		out	dx,ax		; imposta registro maschera di bit
		and	es:[di],bl	; imposta pixel per ultimo byte -1

		mov	ax,VAREndMaskR	; maschera per ultimo byte nella riga di pixel
		out	dx,ax		; .. ultimo byte nella riga di pixel
		and	es:[di+1],bh	; imposta pixel per ultimo byte

		pop	bx		; BX := byte per riga di pixel
		pop	si
		add	si,bx		; DS:SI -> riga successiva nel blocco di bit
		pop	di
		add	di,BytesPerRow	; ES:DI -> successiva riga di pixel nel buffer
		dec	VARRowCounter
		jnz	L18		; loop verso il basso su righe di pixel

		pop	di		; ES:DI -> offset del buffer del video di x,y
		pop	ax		; AX := valore corrente maschera di mappa
		mov	dl,0C4h		; DX := 3C4H
		shr	ah,1		; AH := valore successivo maschera di mappa
		jnz	L17		; loop su piani di bit

; riporta controller grafico e sequencer a loro stati di default

Lexit:		mov	ax,0F02h	; valore di default di maschera di mappa
		out	dx,ax

		mov	dl,0CEh		; DX := 3CEh
		mov	ax,0003		; rotazione dati/selezione funzione di default
		out	dx,ax

		mov	ax,0005		; valore di modalit di default
		out	dx,ax

		mov	ax,0F07h	; valore di confronto colore di default
		out	dx,ax

		mov	ax,0FF08h	; valore di maschera di bit di default
		out	dx,ax

		pop	di		; ripristina registri ed esce
		pop	si
		pop	ds
		mov	sp,bp
		pop	bp
		ret

_StoreBitBlock10 ENDP

_TEXT		ENDS

		END
