                TITLE   'Listato 118'
                NAME    VRMCGA
                PAGE    55,132

;
; Nome:         VRMCGA
;
; Funzione:     Routine di servizio interrupt verticale per MCGA
;
; Chiamante:    Microsoft C:
;
;                       int EnableISR0A();      /* ritorna 0 se installato bene */
;
;                       azzera DisableISR0A();
;

ADDR_6845       EQU     63h

DGROUP          GROUP   _DATA

_TEXT           SEGMENT byte public 'CODE'
                ASSUME  cs:_TEXT,ds:DGROUP

ISR0A           PROC    far             ; gestore interrupt per INT 0Ah

                push    ax              ; mantiene registri
                push    dx
                push    ds

                mov     ax,seg DGROUP
                mov     ds,ax           ; DS -> DGROUP

                mov     dx,Port3x4      ; DX := numero reg indirizzo CRTC
                in      al,dx
                push    ax              ; mantiene valore reg indirizzo CRTC

; determina se si  verificato un interrupt verticale

                mov     al,IConReg      ; AL := numero registro
                out     dx,al
                jmp     $+2             ; attende risposta da MCGA
                inc     dx              ; DX := 3D5H
                in      al,dx           ; AL := valore corrente registro controllo interrupt
                dec     dx
                test    al,40h          ; test bit 6
                jnz     L10             ; salta se si verifica interrupt verticale

; non  un interrupt verticale quindi si collega al precedente gestore di interrupt

                pushf                   ; simula un'INT al
                call    ds:PrevISR0A    ;  precedente gestore di INT 0Ah
                jmp     short Lexit

; gestisce un interrupt verticale

L10:            mov     ax,DefaultICont ; AH := valore di default per reg controllo interrupt
                                        ; AL := 11h (numero reg.)
                and     ah,11101111b    ; AH bit 4 := 0 (cancella latch di interrupt)
                out     dx,ax           ; aggiorna reg. controllo interrupt
                jmp     $+2             ; attende che risponda l'MCGA

; invia fine interrupt al controller di interrupt programmabile
;  per permettere l'emissione in successione di interrupt IRQ2

                mov     al,20h          ; porta di I/O del PIC
                out     20h,al          ; invia fine interrupt non specifico al PIC
                jmp     $+2             ; attende che risponda il PIC
                sti                     ; abilita interrupt

; esegue operazioni utili ...

                inc     word ptr _VRcount       ; incrementa un contatore

; abilita CRTC in modo che generi un altro interrupt

                cli                     ; disabilita interrupt
                mov     ax,DefaultICont ; AH := valore di default del reg controllo interrupt
                                        ; AL := 11h (numero reg.)
                and     ah,11011111b    ; AH bit 5 := 0 (abilita int verticale)
                or      ah,00010000b    ; AH bit 4 := 1 (abilita latch di int)
                out     dx,ax
                jmp     $+2

Lexit:          pop     ax
                out     dx,al           ; ripristina precedente valore 3D4H

                pop     ds              ; ripristina registri ed esce
                pop     dx
                pop     ax
                iret

ISR0A           ENDP

;
; EnableISR0A -- abilita gestore interrupt verticale
;
                PUBLIC  _EnableISR0A
_EnableISR0A    PROC    near

                push    bp              ; mantiene registri del chiamante
                mov     bp,sp
                push    si
                push    di

                mov     ax,40h
                mov     es,ax           ; ES -> area dati BIOS del video

; salva valori di default del registro del CRTC

                mov     dx,es:[ADDR_6845]  ; DX := porta indirizzo del CRTC
                mov     Port3x4,dx      ; salva indirizzo di porta

                mov     ax,1A00h        ; AH := 1AH (numero funzione INT 10H)
                                        ; AL := 0 (legge combinazione di visualizzazione)
                int     10h             ; AL := 1AH se funzione 1AH  supportata
                                        ; BL := sottosistema di visualizzazione attivo
                cmp     al,1Ah
                jne     L20             ; salta se non  MCGA

                cmp     bl,0Bh
                je      L21             ; salta se  MCGA

                cmp     bl,0Ch
                je      L21             ; salta se  MCGA

L20:            mov     ax,0FFFFh       ; ritorna 0FFFFh se non  MCGA
                jmp     short L23

; rileva valore di default per registro controllo interrupt MCGA

L21:            mov     al,IContReg     ; AL := numero reg. controllo interrupt
                cli
                out     dx,al
                jmp     $+2
                inc     dx              ; DX := 3D5H
                in      al,dx           ; AL := valore corrente per il registro
                sti

                mov     IContValue,al   ; salva questo valore

; salva vecchio vettore di interrupt 0Ah

                mov     ax,350Ah        ; AH := 35H (numero funzione INT 21h)
                                        ; AL := 0AH (numero interrupt)
                int     21h             ; ES:BX := precedente vettore INT 0AH

                mov     word ptr PrevISR0A,bx
                mov     word ptr PrevISR0A+2,es ; salva vettore precedente

; aggiorna vettore di interrupt 0AH con indirizzo di questo gestore

                push    ds              ; mantiene DS
                mov     dx,offset ISR0A
                push    cs
                pop     ds              ; DS:DX -> ISR0A
                mov     ax,250Ah        ; AH := 25H (numero funzione INT 21H)
                                        ; AL := 0AH (numero interrupt)
                int     21h             ; aggiorna vettore INT 0AH
                pop     ds              ; ripristina DS

; abilita IRQ2 azzerando il bit 2 del registro di maschera del PIC

                cli                     ; azzera interrupt
                mov     dx,21h          ; DX := registro di maschera del PIC
                in      al,dx           ; AL := valore registro di maschera
                and     al,11111011b    ; azzera bit 2
                out     dx,al

; abilita interrupt verticali

                mov     dx,Port3x4      ; DX := porta indirizzo del CRTC
                mov     ax,DefaultIcont

                and     ah,11001111b
                out     dx,ax           ; azzera bit 4 e 5 del reg controllo int
                jmp     $+2             ; attende risposta dall'MCGA
                or      ah,00010000b
                out     dx,ax           ; imposta bit 4
                jmp     $+2
                sti                     ; abilita interrupt

                xor     ax,ax           ; AX := 0 (ritorna valore)

L23:            pop     di              ; ripristina registri ed esce
                pop     si
                mov     sp,bp
                pop     bp
                ret

_EnableISR0A    ENDP

;
; Disabilita ISR0A -- disabilita gestore interrupt verticale
;
                PUBLIC  _DisableISR0A
_DisableISR0A   PROC    near

                push    bp
                mov     bp,sp
                push    si
                push    di
                push    ds

; disabilita interrupt verticali

                cli                     ; disabilita interrupt
                mov     dx,Port3x4
                mov     ax,DefaultICont
                out     dx,ax           ; ripristina reg controllo interrupt
                jmp     $+2
                sti                     ; abilita interrupt

; ripristina precedente gestore interrupt 0Ah

                lds     dx,PrevISR0A    ; DS:DX := precedente vettore INT 0AH
                mov     ax,250Ah        ; AH := 25H (numero funzione INT 21H)
                                        ; AL := 0AH (numero interrupt)
                int     21h

                pop     ds              ; ripristina registri ed esce
                pop     di
                pop     si
                mov     sp,bp
                pop     bp
                ret

_DisableISR0A   ENDP

_TEXT           ENDS

_DATA           SEGMENT word public 'DATA'

                EXTRN   _VRcount:word   ; dichiarata nel chiamante C

PrevISR0A       DD      ?               ; salva area per vecchio vettore int 0Ah
Port3x4         DW      ?               ; 3B4h o 3D4h

DefaultICont    LABEL   word
IContReg        DB      11h             ; numero reg controllo interrupt
IContValue      DB      ?               ; valore di default per reg controllo interrupt

_DATA           ENDS

                END
