		TITLE	'Listato 119'
		NAME	ScreenOrigin
		PAGE	55,132

;
; Nome:		ScreenOrigin
;
; Funzione:	Imposta origine di schermo su EGA e VGA
;
; Chiamante:	Microsoft C:
;
;			azzera ScreenOrigin(x,y);
;
;				int	x,y;	/* coordinate di pixel x,y */
;

ARGx		EQU	[bp+4]
ARGy		EQU	[bp+6]


CRT_MODE	EQU	49h		; indirizzi nell'area dati del BIOS del video
ADDR_6845	EQU	63h
POINTS		EQU	85h
BIOS_FLAGS	EQU	89h

DGROUP		GROUP	_DATA

_TEXT		SEGMENT	byte public 'CODE'
		ASSUME	cs:_TEXT,ds:DGROUP

		PUBLIC	_ScreenOrigin
_ScreenOrigin	PROC	near

		push	bp		; mantiene registri del chiamante
		mov	bp,sp
		push	si
		push	di

		mov	ax,40h
		mov	es,ax		; ES -> area dati del BIOS del video
		mov	cl,es:[CRT_MODE]

		mov	ax,ARGx		; AX := coordinata x del pixel
		mov	bx,ARGy		; BX := coordinata y del pixel

		cmp	cl,7
		ja	L01		; salta se  in modalit grafica

		je	L02		; salta se  in modal. alfan. monocrom.

		test	byte ptr es:[BIOS_FLAGS],1
		jnz	L02		; salta se  VGA

		jmp	short L03

; impostazione per modalit grafiche (8 pixel per byte)

L01:		mov	cx,8		; CL := 8 (pixel visualizzati per byte)
					; CH := 0 (valore per scansione riga preimpostata)
		div	cl		; AH := offset di bit nel byte
					; AL := offset di byte nella riga di pixel
		mov	cl,ah		; CL := offset di bit (per scorrimento pixel oriz.)
		xor	ah,ah
		xchg	ax,bx		; AX := y
					; BX := offset di byte nella riga di pixel

		mul	word ptr _BytesPerRow
					; AX := offset di byte dell'inizio riga
		jmp	short L05

; impostazione per modalit alfanumeriche VGA e per modalit alfanumerica monocromatica EGA
;  (9 pixel per byte)

L02:					; routine per modalit alfanumeriche
		mov	cx,9		; CL := 9 (pixel visualizzati per byte)
                                        ; CH := 0
		div	cl		; AH := offset di bit nel byte
					; AL := offset di byte nella riga di pixel
		dec	ah		; AH := -1, 0-7
		jns	L04		; salta se offset di bit  da 0 a 7
		mov	ah,8		; AH := 8
		jmp	short L04

; impostazione per modalit alfanumeriche a colori EGA (8 pixel per byte)

L03:		mov	cx,8		; CL := 8 (pixel visualizzati per byte)
					; CH := 0
		div	cl		; AH := offset di bit nel byte
					; AL := offset di byte nella riga di pixel

L04:		mov	cl,ah		; CL := valore per reg scorrimento pixel oriz.
		xor	ah,ah
		xchg	ax,bx		; AX := y
					; BX := offset di byte nella riga
		div	byte ptr es:[POINTS]  ; AL := riga di caratteri
					      ; AH := linea di scansione nella matrice di car.
		xchg	ah,ch		; AX := riga di carattere
					; CH := linea di scansione (valore per reg
					; scansione riga preimpostata)
		mul	word ptr _BytesPerRow	; AX := offset di byte della riga di car.
		shr	ax,1		; AX := offset di word di riga di caratteri

L05:		call	SetOrigin

		pop	di		; ripristina registri ed esce
		pop	si
		mov	sp,bp
		pop	bp
		ret

_ScreenOrigin	ENDP


SetOrigin	PROC	near		; Chiamante:  AX = offset della riga di caratteri
					;              BX = offset di byte nella riga
					;              CH = valore di scansione riga preimpostata
					;              CL = valore di scorrimento pixel orizzont.

		add	bx,ax		; BX := offset di buffer

		mov	dx,es:[ADDR_6845]  ; porta di I/O del CRTC (3B4H o 3D4H)
		add	dl,6		; porta di stato del video (3BAH o 3DAH)

; aggiorna registri inizio indirizzo alto e basso

L20:		in	al,dx		; attende inizio ritracc. verticale
		test	al,8
		jz	L20

L21:		in	al,dx		; attende fine ritracciamento verticale
		test	al,8
		jnz	L21

		cli			; disabilita interrupt
		sub	dl,6		; DX := 3B4H o 3D4H

		mov	ah,bh		; AH := valore di inizio indirizzo alto
		mov	al,0Ch		; AL := numero reg inizio indiriz. alto
		out	dx,ax		; aggiorna questo registro

		mov	ah,bl		; AH := valore per inizio indir. basso
		inc	al		; AL := numero reg. inizio indir. basso
		out	dx,ax		; aggiorna questo registro
		sti			; abilita interrupt

		add	dl,6		; DX := porta di stato del video

L22:		in	al,dx		; attende inizio ritracc. verticale
		test	al,8
		jz	L22

		cli			; disabilita interrupt

		sub	dl,6		; DX := 3B4H o 3D4H
		mov	ah,ch		; AH := valore del reg. scansione riga preimpostata
		mov	al,8		; AL := numero reg scansione riga preimpostata
		out	dx,ax		; aggiorna questo registro

		mov	dl,0C0h		; DX := 3C0h (porta del controller di attributo)
		mov	al,13h OR 20h	; AL bit 0-4 := numero reg. scorrimento pixel oriz.
					; AL bit 5 := 1
		out	dx,al		; scrive reg indirizzo controller di attributo
                                        ;  (flip-flop indirizzo del controller di
                                        ;    attributo  stato azzerato da
                                        ;    IN a L22.)
		mov	al,cl		; AL := valore del reg. scorrimento pixel oriz.
		out	dx,al		; aggiorna questo registro

		sti			; riabilita interrupt
		ret

SetOrigin	ENDP

_TEXT		ENDS


_DATA		SEGMENT	word public 'DATA'

		EXTRN	_BytesPerRow:word	; byte per riga di pixel

_DATA		ENDS

		END
