		TITLE	'Listato 70'
		NAME	LineInC
		PAGE	55,132

;
; Nome:		LineInC
;
; Funzione:	Traccia una linea nella modalit a 16 colori 720x348 Hercules
;
; Chiamante:	Microsoft C:
;
;			azzera LineInC (x1,y1,x2,y2,n);
;
;			int x1,y1,x2,y2;	/* coordinate del pixel */
;
;			int n;			/* valore di pixel */
;

ARGx1		EQU	word ptr [bp+4]	; indirizzamento cornice di stack
ARGy1		EQU	word ptr [bp+6]
ARGx2		EQU	word ptr [bp+8]
ARGy2		EQU	word ptr [bp+10]
ARGn		EQU	byte ptr [bp+12]
VARleafincr	EQU	word ptr [bp-2]
VARincr1	EQU	word ptr [bp-4]
VARincr2	EQU	word ptr [bp-6]
VARroutine	EQU	word ptr [bp-8]

ByteOffsetShift	EQU	3		; usato per convertire i pixel in offset di byte
DefaultRWColor	EQU	0Fh		; valore di default per registro colore lettura/scrittura

_TEXT		SEGMENT	byte public 'CODE'
		ASSUME	cs:_TEXT

		EXTRN	PixelAddrHGC:near

		PUBLIC	_LineInC
_LineInC		PROC	near

		push	bp		; mantiene registri del chiamante
		mov	bp,sp
		sub	sp,8		; spazio di stack per le variabili locali
		push	si
		push	di


		mov	si,2000h	; incremento per intercalazione di buffer del video
		mov	di,90-8000h	; incremento dall'ultima alla prima intercalazione

; imposta registri del CRTC di InColor

		mov	dx,3B4h		; DX := porta di I/O del CRTC
		mov	ax,5F19h	; AH bit 6 := 1 (polarit di maschera)
					; AH bit 5-4 := 1 (modalit di scrittura)
					; AH bit 3-0 := bit "ininfluenti"
					; AL := numero registro di controllo lettura/scrittura
		out	dx,ax	; imposta registro di controllo lettura/scrittura

		inc	ax		; AL := 1Ah (numero registro colore lettura/scrittura)

		mov	ah,ARGn	; AH := valore di primo piano
		out	dx,ax	; imposta registro colore lettura/scrittura

		mov	cx,ARGx2
		sub	cx,ARGx1	; CX := x2 - x1
		jz	VertLineInC	; salta se linea verticale

; forza x1 < x2

		jns	L01		; salta se x2 > x1

		neg	cx		; CX := x1 - x2
		
		mov	bx,ARGx2	; scambia x1 e x2
		xchg	bx,ARGx1
		mov	ARGx2,bx

		mov	bx,ARGy2	; scambia y1 e y2
		xchg	bx,ARGy1
		mov	ARGy2,bx

; calcola dy = ABS(y2-y1)

L01:		mov	bx,ARGy2
		sub	bx,ARGy1	; BX := y2 - y1
		jz	HorizLineInC	; salta se linea orizzontale

		jns	L03

		neg	bx		; BX := y1 - y2
		neg	si		; nega incrementi per intercalazione buffer
		neg	di

; seleziona routine appropriata per inclinazione linea

L03:		mov	VARleafincr,di	; salva incremento per intercalazione buffer

		mov	VARroutine,offset LoSlopeLineInC
		cmp	bx,cx
		jle	L04		; salta se dy <= dx (inclinazione <= 1)
		mov	VARroutine,offset HiSlopeLineInC
		xchg	bx,cx		; scambia dy e dx

; calcola variabile decisionale iniziale e incrementa

L04:		shl	bx,1		; BX := 2 * dy
		mov	VARincr1,bx	; incr1 := 2 * dy
		sub	bx,cx
		mov	di,bx		; DI := d = 2 * dy - dx
		sub	bx,cx
		mov	VARincr2,bx	; incr2 := 2 * (dy - dx)

; calcola primo indirizzo di pixel

		push	cx		; mantiene questo registro
		mov	ax,ARGy1	; AX := y
		mov	bx,ARGx1	; BX := x
		call	PixelAddrHGC	; AH := maschera di bit
					; ES:BX -> buffer
					; CL := # bit da spostare a sinistra

		shl	ah,cl
		mov	dh,ah		; DH := maschera di bit nella posizione corretta

		pop	cx		; ripristina questo registro
		inc	cx		; CX := # di pixel da tracciare

		jmp	VARroutine	; salta a routine appropriata per inclinazione

; routine per linee verticali

VertLineInC:	mov	ax,ARGy1	; AX := y1
		mov	bx,ARGy2	; BX := y2
		mov	cx,bx
		sub	cx,ax		; CX := dy
		jge	L31		; salta se dy >= 0

		neg	cx		; forza dy >= 0
		mov	ax,bx		; AX := y2

L31:		inc	cx		; CX := # di pixel da tracciare
		mov	bx,ARGx1	; BX := x
		push	cx		; mantiene questo registro
		call	PixelAddrHGC	; AH := maschera di bit
					; ES:BX -> buffer del video
					; CL := # bit da spostare a sinistra
		shl	ah,cl		; AH := maschera di bit nella posizione corretta
		pop	cx		; ripristina questo registro

L32:		or	es:[bx],ah	; imposta valori di pixel nel buffer

		add	bx,si		; incrementa all'area successiva di intercalazione
		jns	L33

		add	bx,di		; incrementa alla prima area d'intercalazione
L33:		loop	L32

		jmp	Lexit


; routine per linee orizzontali (inclinazione = 0)

HorizLineInC:	mov	ax,ARGy1
		mov	bx,ARGx1
		call	PixelAddrHGC	; AH := maschera di bit
					; ES:BX -> buffer del video
					; CL := # bit da spostare a sinistra
		mov	di,bx		; ES:DI -> buffer

		mov	dh,ah
		not	dh		; DH := maschera di bit non spostata per
					;        byte all'estrema sinistra
		mov	dl,0FFh		; DL := maschera di bit non spostata per
					;	 byte all'estrema destra

		shl	dh,cl		; DH := inverte maschera di bit per primo byte
		not	dh		; DH := maschera di bit per primo byte

		mov	cx,ARGx2
		and	cl,7
		xor	cl,7		; CL := numero di bit da spostare a sinistra
		shl	dl,cl		; DL := maschera di bit per ultimo byte

; determina offset byte del primo e dell'ultimo pixel della linea

		mov	ax,ARGx2	; AX := x2
		mov	bx,ARGx1	; BX := x1

		mov	cl,ByteOffsetShift	; numero di bit da spostare per
						;  convertire i pixel in byte

		shr	ax,cl		; AX := offset di byte di x2
		shr	bx,cl		; BX := offset di byte di x1
		mov	cx,ax
		sub	cx,bx		; CX := (# byte nella linea) - 1

; imposta pixel nel byte all'estrema sinistra della linea

		or	dh,dh
		js	L43		; salta se  allineato al byte (x1  il pixel
					;   all'estrema sinistra nel byte)
		or	cx,cx
		jnz	L42		; salta se c' pi di un byte nella linea

		and	dl,dh		; maschera di bit per la linea
		jmp	short L44

L42:		or	es:[di],ah	; aggiorna pixel mascherati nel buffer
		inc	di
		dec	cx

; usa una veloce istruzione macchina 8086 per tracciare il resto della linea

L43:		mov	al,0FFh		; maschera di bit a 8 pixel
		rep	stosb		; aggiorna tutti i pixel nella linea

; imposta pixel nel byte all'estrema destra della linea

L44:		or	es:[di],al	; aggiorna pixel mascherati nel buffer
		jmp	Lexit


; routine per dy <= dx (inclinazione <= 1)	; ES:BX -> buffer del video
                                        ; CX = # pixel da tracciare
					; DH = maschera di bit
					; SI = incremento dell'intercalazione del buffer
					; DI = variabile decisionale
LoSlopeLineInC:

L10:		mov	ah,es:[bx]	; piani di bit di latch
					; AH := 0 perch tutti i piani sono "ininfluenti"

L11:		or	ah,dh		; imposta valore di pixel nel byte

		ror	dh,1		; ruota maschera di bit
		jc	L14		; salta se maschera di bit  stata ruotata sulla
					;  posizione di pixel all'estrema sinistra

; maschera di bit non spostata fuori

		or	di,di		; test del segno di d
		jns	L12		; salta se d >= 0

		add	di,VARincr1	; d := d + incr1
		loop	L11

		mov	es:[bx],ah	; memorizza restanti pixel nel buffer
		jmp	short Lexit

L12:		add	di,VARincr2	; d := d + incr2
		mov	es:[bx],ah	; aggiorna buffer

		add	bx,si		; incrementa y
		jns	L13		; salta se non si trova nell'ultima intercalazione

		add	bx,VARleafincr	; incrementa nell'intercalazione successiva

L13:		loop	L10
		jmp	short Lexit

; maschera di bit spostata fuori

L14:		mov	es:[bx],ah	; aggiorna buffer
		inc	bx		; BX := offset del byte successivo

		or	di,di		; test del segno di d
		jns	L15		; salta se non negativo

		add	di,VARincr1	; d := d + incr1
		loop	L10
		jmp	short Lexit

L15:		add	di,VARincr2	; d := d + incr2

		add	bx,si		; incrementa y
		jns	L16		; salta se non si trova nell'ultima intercalazione

		add	bx,VARleafincr	; incrementa nell'intercalazione successiva

L16:		loop	L10		; loop fino a che tutti i pixel sono impostati
		jmp	short Lexit


; routine per dy > dx (inclinazione > 1)	; ES:BX -> buffer del video
                                        ; CX = # pixel da tracciare
					; DH = maschera di bit
					; SI = incremento dell'intercalazione del buffer
					; DI = variabile decisionale
HiSlopeLineInC:

L21:		or	es:[bx],dh	; imposta valore di pixel nel buffer del video

		add	bx,si		; incrementa y
		jns	L22		; salta se non si trova nell'ultima intercalazione

		add	bx,VARleafincr	; incrementa nell'intercalazione successiva

L22:		or	di,di
		jns	L23		; salta se d >= 0

		add	di,VARincr1	; d := d + incr1
		loop	L21
		jmp	short Lexit


L23:		add	di,VARincr2	; d := d + incr2

		ror	dh,1		; ruota maschera di bit
		adc	bx,0		; BX := offset del byte successivo (incrementato
					;  se maschera di bit  stata ruotata sulla
					;  posizione all'estrema sinistra)

		loop	L21


Lexit:		mov	dx,3B4h	; DX := porta di I/O del CRTC
		mov	ax,0F18h
		out	dx,ax		; ripristina valore di default di maschera di piano

		mov	ax,4019h		; ripristina valore di default di controllo lettura/scrittura
		out	dx,ax

		inc	ax		; ripristina valore di default di colore lettura/scrittura
		mov	ah,DefaultRWColor
		out	dx,ax

		pop	di		; ripristina registri e ritorna
		pop	si
		mov	sp,bp
		pop	bp
		ret

_LineInC	ENDP

_TEXT		ENDS

		END
