		TITLE	'Listato 143'
		NAME	ScreenDumpCGA
		PAGE	55,132

;
; Nome:		ScreenDumpCGA
;
; Funzione:	Stampa di schermo per le modalit a 2 colori 640x200 e a 4 colori 320x200 CGA
;
; Chiamante:	(non deinito)
;
; Note:		La procedura principale di questo programma, ScreenDumpCGA, pu essere
;		chiamata  da un programma applicativo o come parte di un gestore TSR
;		(concludi ma resta residente) handler per l'interrupt 5.
;

STDPRN		=	4		; handle standard di stampante MS-DOS


DGROUP		GROUP	_DATA

_TEXT		SEGMENT	byte public 'CODE'
		ASSUME	cs:_TEXT,ds:DGROUP

;
; PrintLine
;
; 	Scrive una linea di caratteri sul dispositivo di stampante standard.  Ignora
;	errori.
;

PrintLine	PROC	near		; Chiamante:	DS:DX -> dati
					;		CX = # di byte
		mov	bx,STDPRN
		mov	ah,40h		; funzione 40h INT 21h:  scrive
		int	21h
		ret

PrintLine	ENDP

;
; PrinterGraphics
;
;	Attiva la modalit grafica della stampante.  Questa routine deve essere adattata alle
;	diverse stampanti.
;

PrinterGraphics	PROC	near		; configura stampante Epson MX-80
					;  per 480 punti/linea

		mov	dx,offset DGROUP:EpsonGraphics
		mov	cx,3
		call	PrintLine
		ret

PrinterGraphics	ENDP

;
; PrinterDefault
;
;	Attiva la modalit di default della stampante (non grafica).  Anche in questo caso,
;	questa routine deve essere adattata alle diverse stampanti.
;

PrinterDefault	PROC	near		; configura Epson MX-80 per output
					;  alfanumerico di default

		mov	dx,offset DGROUP:EpsonReset
		mov	cx,2
		call	PrintLine
		ret

PrinterDefault	ENDP

;
; ChopZeroes
;
;	Elimina zero di coda dal buffer di output della stampante.
;

ChopZeroes	PROC	near		; Chiamante:	ES:DI -> buffer
					;		CX = lunghezza buffer
					; Ritorna:	CX = lunghezza modificata

		jcxz	L01		; esce se il buffer  vuoto

		add	di,cx
		dec	di		; ES:DI -> ultimo byte nel buffer

		xor	al,al		; AL := 0 (byte da ricercare in scansione)

		std			; scansione all'indietro
		repe	scasb
		cld			; ripristina flag di direzione
		je	L01		; salta se il buffer  pieno di zeri

		inc	cx		; modifica lunghezza oltre ultimo byte non zero

L01:		ret

ChopZeroes	ENDP

;
; PrintPixels
;
;	Stampa una riga di pixel su un'Epson MX-80.
;

PrintPixels	PROC	near		; Chiamante:	DI = offset del buffer
					;		CX = lunghezza del buffer

		push	ds
		pop	es		; ES := DS

		push	di		; mantiene offset del buffer
		call	ChopZeroes
		push	cx		; mantiene lunghezza

		mov	word ptr DataHeader+2,cx  ; memorizza lunghezza del buffer
						  ;  nell'intestazione dati di output
		mov	dx,offset DGROUP:DataHeader
		mov	cx,4
		call	PrintLine	; stampa intestazione dati

		pop	cx		; CX := lunghezza buffer
		pop	dx		; DX := offset del buffer
		call	PrintLine	; stampa i pixel

		mov	dx,offset DGROUP:CRLF
		mov	cx,2
		call	PrintLine

		ret

PrintPixels	ENDP

;
; TranslatePixels
;
;	Copia una riga stampabile di pixel dal buffer del video al
;	buffer di stampa.  Questa routine pu essere modificata a piacere per modificare
;	la scalatura o la direzione dell'immagine stampata, per inserire valori di
;	scala di grigi per i pixel a colori, ecc.
;
;	Questa routine formatta il buffer della stampante per l'output su una Epson
;	MX-80.  La pagina viene stampata verticalmente, con due pixel stampati orizzontalmente
;	per ogni pixel verticale nel buffer del video.  Dal momento che lo schermo CGA
;	 alto 200 pixel, l'output di stampa  largo 400 pixel.
;

TranslatePixels	PROC	near		; Chiamante:	SI = offset del buffer del video
					;		ES:DI -> buffer di stampa

		push	ds		; matiene DS
		mov	ds,VideoBufSeg	; DS:SI -> buffer del video

		add	di,398		; ES:DI -> 2 byte prima della fine del buffer

		mov	cx,200		; CX := # di pixel verticali
		mov	bx,2000h+1	; BX := primo incremento del buffer del video
		mov	dx,81-2000h	; DX := secondo incremento del buffer del video

		std			; riempie il buffer di stampa all'indietro

L11:		lodsb			; AL := 8 pixel dal buffer del video
		mov	ah,al		; AX := 8 pixel raddoppiati
		stosw			; li scrive sul buffer di stampa

		add	si,bx		; incremento alla successiva intercalazione del
		xchg	bx,dx		;  buffer del video

		loop	L11

		cld			; azzera flag di direzione
		pop	ds		; ripristina DS
		ret

TranslatePixels	ENDP

;
; ScreenDumpCGA
;

ScreenDumpCGA	PROC	near		; Chiamante:	DS = DGROUP

		call	PrinterGraphics	; configura la stampante per la grafica

		push	ds
		pop	es		; DS,ES := DGROUP

		xor	si,si		; SI := offset dell'inizio del buffer del video

L21:		push	si
		mov	di,offset DGROUP:PrintBuf
		call	TranslatePixels	; copia una riga stampabile di pixel

		mov	cx,400
		mov	di,offset DGROUP:PrintBuf
		call	PrintPixels	; li stampa

		pop	si
		inc	si
		cmp	si,80		; loop di tutte le 80 colonne del
		jb	L21		;  buffer del video
				
		call	PrinterDefault	; riporta la stampante al suo stato
					;  di default
		ret

ScreenDumpCGA	ENDP

_TEXT		ENDS


_DATA		SEGMENT	word public 'DATA'

PrintBuf	DB	400 dup(?)	; stampa buffer di output

VideoBufSeg	DW	0B800h

EpsonGraphics	DB	1Bh,33h,18h
EpsonReset	DB	1Bh,40h
DataHeader	DB	1Bh,4Bh,00h,00h
CRLF		DB	0Dh,0Ah

_DATA		ENDS

		END
