		TITLE	'Listato 144'
		NAME	ScreenDumpEGA
		PAGE	55,132

;
; Nome:		ScreenDumpEGA
;
; Funzione:	Stampa di schermo per la modalit a 16 colori 640x350 EGA
;
; Chiamante:	(non definito)
;
; Note:		La procedura principale di questo programma, ScreenDumpEGA, pu essere
;		chiamata  da un programma applicativo o come parte di un gestore TSR
;		(concludi ma resta residente) per l'interrupt 5.
;

STDPRN		=	4		; handle standard di stampante MS-DOS


DGROUP		GROUP	_DATA

_TEXT		SEGMENT	byte public 'CODE'
		ASSUME	cs:_TEXT,ds:DGROUP

;
; PrintLine
;
; 	Scrive una linea di caratteri sul dispositivo di stampante standard. Ignora
;	errori.
;

PrintLine	PROC	near		; Chiamante:	DS:DX -> dati
					;		CX = # di byte
		mov	bx,STDPRN
		mov	ah,40h		; funzione 40h INT 21h:  scrive
		int	21h
		ret

PrintLine	ENDP

;
; PrinterGraphics
;
;	Attiva la modalit grafica della stampante. Questa routine deve essere adattata alle
;	diverse stampanti .
;

PrinterGraphics	PROC	near		; configura stampante Epson MX-80
					;  per 480 punti/linea

		mov	dx,offset DGROUP:EpsonGraphics
		mov	cx,3
		call	PrintLine
		ret

PrinterGraphics	ENDP

;
; PrinterDefault
;
;	Attiva la modalit di default della stampante (non grafica).  Anche in questo caso,
;	questa routine deve essere adattata alle diverse stampanti.
;

PrinterDefault	PROC	near		; configura Epson MX-80 per output
					;  alfanumerico di default

		mov	dx,offset DGROUP:EpsonReset
		mov	cx,2
		call	PrintLine
		ret

PrinterDefault	ENDP

;
; ChopZeroes
;
;	Elimina zeri di coda dal buffer di output della stampante.
;

ChopZeroes	PROC	near		; Chiamante:	ES:DI -> buffer
					;		CX = lunghezza buffer
					; Ritorna:	CX = lunghezza modificata

		jcxz	L01		; esce se il buffer  vuoto

		add	di,cx
		dec	di		; ES:DI -> ultimo byte nel buffer

		xor	al,al		; AL := 0 (byte da ricercare in scansione)

		std			; scansione all'indietro
		repe	scasb
		cld			; ripristina flag di direzione
		je	L01		; salta se il buffer  pieno di zeri

		inc	cx		; modifica lunghezza oltre ultimo byte non zero

L01:		ret

ChopZeroes	ENDP

;
; PrintPixels
;
;	Stampa una riga di pixel su un'Epson MX-80.
;

PrintPixels	PROC	near		; Chiamante:	DI = offset del buffer
					;		CX = lunghezza del buffer

		push	ds
		pop	es		; ES := DS

		push	di		; mantiene offset del buffer
		call	ChopZeroes
		push	cx		; mantiene lunghezza

		mov	word ptr DataHeader+2,cx  ; memorizza lunghezza del buffer
						  ;  nell'intestazione dati di output
		mov	dx,offset DGROUP:DataHeader
		mov	cx,4
		call	PrintLine	; stampa intestazione dati

		pop	cx		; CX := lunghezza buffer
		pop	dx		; DX := offset del buffer
		call	PrintLine	; stampa i pixel

		mov	dx,offset DGROUP:CRLF
		mov	cx,2
		call	PrintLine

		ret

PrintPixels	ENDP

;
; TranslatePixels
;
;	Copia una riga stampabile di pixel dal buffer del video al
;	buffer di stampa.  Questa routine pu essere modificata a piacere per modificare
;	la scalatura o la direzione dell'immagine stampata, per inserire valori di
;	scala di grigi per i pixel a colori, ecc.
;
;	Questa routine formatta il buffer della stampante per l'output su una Epson
;	MX-80.  La pagina viene stampata verticalmente, quindi l'output di stampa  largo 350 pixel.
;

TranslatePixels	PROC	near		; Chiamante:	SI = offset del buffer del video
					;		ES:DI -> buffer di stampa

		push	ds		; mantiene DS
		mov	ds,VideoBufSeg	; DS:SI -> buffer del video

		add	di,349		; ES:DI -> ultimo byte nel buffer di stampa

		mov	cx,350		; CX := # di pixel verticali

; imposta il controller grafico per la modalit di lettura 1

		mov	dx,3CEh		; porta di I/O del controller grafico
		mov	ax,805h		; AH := 00001000b (modalit di lettura 1)
					; AL := numero registro modalit
		out	dx,ax

		mov	ax,002		; AH := 0 (valore di confronto colore)
		out	dx,ax		; AL := numero registro confronto colore

		mov	ax,0F07h	; AH := 00001111b (maschera colore ininfluente)
		out	dx,ax		; AL := numero registro colore ininfluente

; riempie il buffer di stampa; tutti i pixel diversi da zero nel buffer del video vengono stampati

		std			; riempie il buffer di stampa all'indietro

L11:		lodsb			; AL := valore di confronto colore a 8 pixel
					;  (bit = 0 se pixel <> 0)
		not	al		; AL := 8 pixel stampabili
		stosb			; memorizza nel buffer di stampa

		add	si,81		; incrementa alla successiva riga nel buffer del video
		loop	L11

		cld			; azzera flag di direzione

; ripristina stato di default del controller grafico

		mov	ax,5		; AH := modalit di lettura 0, modalit di scrittura 0
		out	dx,ax		; AL := numero registro modalit

		mov	ax,7		; AH := 0 (maschera di colore ininfluente)
		out	dx,ax		; AL := numero registro colore ininfluente

		pop	ds		; ripristina DS
		ret

TranslatePixels	ENDP

;
; ScreenDumpEGA
;

ScreenDumpEGA	PROC	near		; Chiamante:	DS = DGROUP

		call	PrinterGraphics	; configura la stampante per la grafica

		push	ds
		pop	es		; DS,ES := DGROUP

		xor	si,si		; SI := offset dell'inizio del buffer del video

L21:		push	si
		mov	di,offset DGROUP:PrintBuf
		call	TranslatePixels	; copia una riga stampabile di pixel

		mov	cx,350
		mov	di,offset DGROUP:PrintBuf
		call	PrintPixels	; li stampa

		pop	si
		inc	si
		cmp	si,80		; loop su tutte le 80 colonne del
		jb	L21		;  buffer del video
				
		call	PrinterDefault	; riporta la stampante al suo stato
					;  di default
		ret

ScreenDumpEGA	ENDP

_TEXT		ENDS


_DATA		SEGMENT	word public 'DATA'

PrintBuf	DB	350 dup(?)	; stampa buffer di output

VideoBufSeg	DW	0A000h

EpsonGraphics	DB	1Bh,33h,18h
EpsonReset	DB	1Bh,40h
DataHeader	DB	1Bh,4Bh,00h,00h
CRLF		DB	0Dh,0Ah

_DATA		ENDS

		END
