		TITLE	'Listato 146'
		NAME	VideoID
		PAGE	55,132

;
; Nome:		VideoID
;
; Funzione:	Rileva la presenza dei vari sottosistemi di visualizzazione
;		e monitor collegati.
;
; Chiamante:	Microsoft C:
;
;			azzera VideoID(VIDstruct);
;
;			struct
;			{
;			  char VideoSubsystem;
;			  char Display;
;			}
;				*VIDstruct[2];
;
;		valori ID sottosistema:
;				 0  = (nessuno)
;				 1  = MDA
;				 2  = CGA
;				 3  = EGA
;				 4  = MCGA
;				 5  = VGA
;				80h = HGC
;				81h = HGC+
;				82h = scheda InColor Hercules
;
;		tipi di monitor: 0  = (nessuno)
;				 1  = monocromatico compatibile MDA
;				 2  = a colori compatibile CGA
;				 3  = a colori compatibile EGA
;				 4  = monocromatico compatibile PS/2
;				 5  = a colori compatibile PS/2
;				 
;
;	I valori ritornati in VIDstruct[0].VideoSubsystem e
;	VIDstruct[0].Display indicano il sottosistema attivo correntemente.
;

ARGpVID		EQU	word ptr [bp+4]	; indirizzamento cornice di stack


VIDstruct	STRUC			; corrisponde a struttura dati in C

Video0Type	DB	?		; primo tipo di sottositema
Display0Type	DB	? 		; monitor collegato al primo sottosistema

Video1Type	DB	?		; secondo tipo di sottosistema
Display1Type	DB	?		; monitor collegato al secondo sottosistema

VIDstruct	ENDS


Device0		EQU	word ptr Video0Type[di]
Device1		EQU	word ptr Video1Type[di]


MDA		EQU	1		; tipi di sottosistemi
CGA		EQU	2
EGA		EQU	3
MCGA		EQU	4
VGA		EQU	5
HGC		EQU	80h
HGCPlus		EQU	81h
InColor		EQU	82h

MDADisplay	EQU	1		; tipi di monitor
CGADisplay	EQU	2
EGAColorDisplay	EQU	3
PS2MonoDisplay	EQU	4
PS2ColorDisplay	EQU	5

TRUE		EQU	1
FALSE		EQU	0


DGROUP		GROUP	_DATA

_TEXT		SEGMENT	byte public 'CODE'
		ASSUME	cs:_TEXT,ds:DGROUP

		PUBLIC	_VideoID
_VideoID	PROC	near

		push	bp		; mantiene registri del chiamante
		mov	bp,sp
		push	si
		push	di

; inizializza la struttura dati che conterr i risultati

		mov	di,ARGpVID	; DS:DI -> inizio della struttura dati

		mov	Device0,0	; azzera queste variabili
		mov	Device1,0

; ricerca i vari sottosistemi usando le subroutine i cui indirizzi sono
;  tabulati in TestSequence; ogni subroutine imposta flag in TestSequence
;  per indicare se subroutine successive devono essere richiamate

		mov	cx,NumberOfTests
		mov	si,offset DGROUP:TestSequence

L01:		lodsb			; AL := flag
		test	al,al
		lodsw			; AX := indirizzo di subroutine
		jz	L02		; salta subroutine se flag  falsa

		push	si
		push	cx
		call	ax		; richiama subroutine per rilevare sottosistema
		pop	cx
		pop	si

L02:		loop	L01

; determina quale sottosistema  attivo

		call	FindActive

		pop	di		; ripristina registri del chiamante e ritorna
		pop	si
		mov	sp,bp
		pop	bp
		ret

_VideoID	ENDP


;
; FindPS2
;
; 	Questa subroutine usa la funzione 1Ah di INT 10H per determinare il codice di 
;	 combinazioni visualizzazioni (DCC) del BIOS del video per ogni sottosistema di
;	 visualizzazione presente.
;

FindPS2		PROC	near

		mov	ax,1A00h
		int	10h		; richiama il BIOS del video per info

		cmp	al,1Ah
		jne	L13		; esce se funzione non supportata (cio
					;  n MCGA n VGA sono nel sistema)

; converte DCC del BIOS in sottositemi e monitor specifici

		mov	cx,bx
		xor	bh,bh		; BX := DCC per sottosistema attivo

		or	ch,ch
		jz	L11		; salta se solo un sottositema  presente

		mov	bl,ch		; BX := DCC inattivo
		add	bx,bx
		mov	ax,[bx+offset DGROUP:DCCtable]

		mov	Device1,ax

		mov	bl,cl
		xor	bh,bh		; BX := DCC attivo

L11:		add	bx,bx
		mov	ax,[bx+offset DGROUP:DCCtable]

		mov	Device0,ax

; azzera flag per sottositemi che sono stati scartati

		mov	byte ptr CGAflag,FALSE
		mov	byte ptr EGAflag,FALSE
		mov	byte ptr Monoflag,FALSE

		lea	bx,Video0Type[di]  ; se il BIOS ha riportato un MDA ...
		cmp	byte ptr [bx],MDA
		je	L12

		lea	bx,Video1Type[di]
		cmp	byte ptr [bx],MDA
		jne	L13

L12:		mov	word ptr [bx],0    ; ... Hercules non pu essere scartato
		mov	byte ptr Monoflag,TRUE

L13:		ret

FindPS2		ENDP


;
; FindEGA
;
; Ricerca un EGA.  Questo viene realizzato effettuando una chiamata ad una funzione del BIOS dell'EGA
;  che non esiste nel BIOS di default (MDA, CGA).

FindEGA		PROC	near		; Chiamante:	AH = flag
					; Ritorna:	AH = flag
					;		Video0Type e
					;		 Display0Type aggiornati

		mov	bl,10h		; BL := 10h (ritorna info su EGA)
		mov	ah,12h		; AH := numero funzione INT 10H
		int	10h		; richiama BIOS EGA per info
					; se BIOS EGA  presente,
					;  BL <> 10H
					;  CL = impostazione commutatore
		cmp	bl,10h
		je	L22		; salta se BIOS EGA non  presente

		mov	al,cl
		shr	al,1		; AL := commutatori/2
		mov	bx,offset DGROUP:EGADisplays
		xlat			; determina tipo monitor da commutatori
		mov	ah,al		; AH := tipo monitor
		mov	al,EGA		; AL := tipo sottosistema
		call	FoundDevice

		cmp	ah,MDADisplay
		je	L21		; salta se EGA ha un monitor monocromatico

		mov	CGAflag,FALSE	; non  CGA se EGA ha monitor a colori
		jmp	short L22

L21:		mov	Monoflag,FALSE	; EGA ha un monitor monocromatico, quindi MDA ed
					;  Hercules sono scartati
L22:		ret

FindEGA		ENDP


;
; FindCGA
;
;	Questo viene realizzato cercando il CRTC 6845 del CGA alla porta di I/O 3D4H.
;

FindCGA		PROC	near		; Ritorna:	VIDstruct aggiornato

		mov	dx,3D4h		; DX := porta indirizzo CRTC
		call	Find6845
		jc	L31		; salta se non presente

		mov	al,CGA
		mov	ah,CGADisplay
		call	FoundDevice

L31:		ret

FindCGA		ENDP


;
; FindMono
;
;	Questo viene realizzato cercando il CRTC 6845 dell'MDA alla porta di I/O 3B4H.  Se
;	un 6845 viene trovato, la subroutine distingue tra un adattatore MDA
;	ed un adattatore Hercules verificando il bit 7 del byte di stato del CRT.
;	Questo bit cambia su adattatori Hercules ma non cambia su MDA.
;
;	I vari adattatori Hercules vengono identificati dai bit da 4 a 6 del
;	valore di stato del CRT:
;
;		000b = HGC
;		001b = HGC+
;		101b = scheda InColor
;

FindMono	PROC	near		; Ritorna:	VIDstruct aggiornato

		mov	dx,3B4h		; DX := porta indirizzo CRTC
		call	Find6845
		jc	L44		; salta se non presente

		mov	dl,0BAh		; DX := 3BAh (porta di stato)
		in	al,dx
		and	al,80h
		mov	ah,al		; AH := bit 7 (sincronizzazione verticale su HGC)

		mov	cx,8000h	; ripete questo 32768 volte
L41:		in	al,dx
		and	al,80h		; isola bit 7
		cmp	ah,al
		loope	L41		; attende modifica bit 7

		jne	L42		; se bit 7 modificato,  un Hercules

		mov	al,MDA		; se bit 7 non cambia,  un MDA
		mov	ah,MDADisplay
		call	FoundDevice
		jmp	short L44

L42:		in	al,dx
		mov	dl,al		; DL := valore da porta di stato

		mov	ah,MDADisplay	; assume sia un monitor monocromatico

		mov	al,HGC		; ricerca un HGC
		and	dl,01110000b	; maschera bit da 4 a 6
		jz	L43

		mov	al,HGCPlus	; ricerca un HGC+
		cmp	dl,00010000b
		je	L43		; salta se  un HGC+

		mov	al,InColor	;  una scheda InColor
		mov	ah,EGAColorDisplay

L43:		call	FoundDevice

L44:		ret

FindMono	ENDP


;
; Find6845
;
;	Questa routine rileva la presenza del CRTC su un MDA, CGA o HGC.
;	La tecnica  quella di scrivere e di leggere il registro 0Fh del chip (cursore
;	basso).  Se lo stesso valore viene letto come scritto, si assume che il chip
;	 presente all'indirizzo di porta specificato.
;

Find6845	PROC	near		; Chiamante:	DX = indirizzo di porta
					; Ritorna:	cf impostato se non presente
		mov	al,0Fh
		out	dx,al		; seleziona reg 0Fh (cursore basso) 6845
		inc	dx
		in	al,dx		; AL := valore corrente di cursore basso
		mov	ah,al		; mantiene in AH
		mov	al,66h		; AL := valore arbitrario
		out	dx,al		; tenta di scrivere su 6845

		mov	cx,100h
L51:		loop	L51		; attende che 6845 risponda

		in	al,dx
		xchg	ah,al		; AH := valroe ritornato
					; AL := valore originale
		out	dx,al		; ripristina valore originale

		cmp	ah,66h		; verifica che 6845 abbia risposto
		je	L52		; salta in caso affermativo (cf viene azzerato)

		stc			; imposta flag riporto se 6845 non  presente

L52:		ret	
	
Find6845	ENDP


;
; FindActive
;
;	Questa subroutine memorizza il dispositivo correntemente attivo come Device0.  La
;	modalit video corrente determina quale sottosistema  attivo.
;

FindActive	PROC	near

		cmp	word ptr Device1,0
		je	L63			; esce se  un solo sottosistema

		cmp	Video0Type[di],4	; esce se MCGA o VGA presente
		jge	L63			;  (funzione 1AH di INT 10H
		cmp	Video1Type[di],4	;  ha gi svolto il compito)
		jge	L63

		mov	ah,0Fh
		int	10h			; AL := modalit video BIOS corrente

		and	al,7
		cmp	al,7			; salta se monocromatico
		je	L61			;  (modalit 7 o 0Fh)

		cmp	Display0Type[di],MDADisplay
		jne	L63			; esce se Display0  a colori
		jmp	short L62

L61:		cmp	Display0Type[di],MDADisplay
		je	L63			; esce se Display0  monocromatico

L62:		mov	ax,Device0		; rende correntemente attivo Device0
		xchg	ax,Device1
		mov	Device0,ax

L63:		ret

FindActive	ENDP


;
; FoundDevice
;
;	Questa routine aggiorna l'elenco dei sottosistemi.
;

FoundDevice	PROC	near			; Chiamante: AH = monitor #
						;            AL = sottosistema #
						; Distrugge: BX
		lea	bx,Video0Type[di]
		cmp	byte ptr [bx],0
		je	L71			; salta se primo sottosistema

		lea	bx,Video1Type[di]	; deve essere secondo sottosistema

L71:		mov	[bx],ax			; aggiorna elemento di elenco
		ret

FoundDevice	ENDP

_TEXT		ENDS


_DATA		SEGMENT	word public 'DATA'

EGADisplays	DB	CGADisplay	; 0000b, 0001b	(valori commutatore EGA)
		DB	EGAColorDisplay	; 0010b, 0011b
		DB	MDADisplay	; 0100b, 0101b
		DB	CGADisplay	; 0110b, 0111b
		DB	EGAColorDisplay	; 1000b, 1001b
		DB	MDADisplay	; 1010b, 1011b

DCCtable	DB	0,0		; traduce tabella per funzione 1Ah INT 10h
		DB	MDA,MDADisplay
		DB	CGA,CGADisplay
		DB	0,0
		DB	EGA,EGAColorDisplay
		DB	EGA,MDADisplay
		DB	0,0
		DB	VGA,PS2MonoDisplay
		DB	VGA,PS2ColorDisplay
		DB	0,0
		DB	MCGA,EGAColorDisplay
		DB	MCGA,PS2MonoDisplay
		DB	MCGA,PS2ColorDisplay

TestSequence    DB      TRUE            ; questo elenco di flag e di indirizzi
		DW	FindPS2		;  determina l'ordine nel quale questo
					;  programma cerca i vari
EGAflag         DB      TRUE            ;  sottositemi
		DW	FindEGA

CGAflag         DB      TRUE
		DW	FindCGA

Monoflag        DB      TRUE
		DW	FindMono

NumberOfTests	EQU	($-TestSequence)/3

_DATA		ENDS

		END
